package INSTALL::common;

use INSTALL::os;
use IO::Handle;
use POSIX;

BEGIN {
  use Exporter;
  
  @ISA = qw(Exporter);
  
  @EXPORT = 
    qw(
       error 
       trace
       note
       report
       indent
       basename
       dirname
       usleep
       mkpath
       realpath
       locate_binary
       find_file
       find_files
      );
}

# Always ignore Ctrl-C.
$main::SIG{INT} = "IGNORE";
$main::ENV{TERM} = "vt100";

### umask 0;
STDOUT->autoflush(1);

my $pOS = \%INSTALL::os::Details;
my $ERROR_LOG = undef;

my %CONST_OutputModes =
  (
   Log 		=> 1,
   Display	=> 2,
   Both		=> 3
  );

#------------------------------------------------------------------------------
# Function takes a pointer to caller info structure (containing a list of 
# package, filename, and line number), a header, then the subsequent lines 
# of output.
#------------------------------------------------------------------------------
my $report_internal = sub
{
  my $pCaller_info = shift @_;
  my $output_mode = shift @_;
  my $header = shift @_;
  my $separator = "\n";
  $separator .= " " x length($header);
  
  my $result = $header . join($separator,@_) . "\n";

  if ($CONST_OutputModes{Display} & $output_mode) {
    print STDERR $result;
  }

  if (($CONST_OutputModes{Log} & $output_mode) && defined $ERROR_LOG) {
    my $time = localtime(time);
    my ($package,$filename,$line) = @$pCaller_info;

    print $ERROR_LOG ("====================================================== $time\n",
		      $result,
		      "Encountered at:\n",
		      "  Package: $package\n", 
		      "  File: $filename\n",
		      "  Line: $line\n",
		      "===============================================================================\n"
		     );
  }

  return(undef);
};

#------------------------------------------------------------------------------
# The report functions take a header, followed by a list of strings to display
# on separate lines past the header:
#
# Example:
# report("Warning: ",
#        "Continuing on with the current course will result in the",
#        "destruction of your server.");
#
#  ==> "Warning: Continuing on with the current course will result in the 
#                destruction of your server."
#
# Note: report displays only to the screen
#       report_log displays only to the log
#       report_both displays to both the log and the screen
#------------------------------------------------------------------------------
sub report
{
  return(&$report_internal([caller],$CONST_OutputModes{Display},@_));
}

sub report_log
{
  return(&$report_internal([caller],$CONST_OutputModes{Log},@_));
}

sub report_both
{
  return(&$report_internal([caller],$CONST_OutputModes{Both},@_));
}

#------------------------------------------------------------------------------
# The error function(s) works similarly to the report function except that the 
# header is implicitly "Error: ".  
#
# Note: error displays to both the log and the screen
#       error_log displays to only the log
#       error_display displays to only the screen
#------------------------------------------------------------------------------
sub error
{
  return(&$report_internal([caller],$CONST_OutputModes{Both},"Error: ",@_));
}

sub error_log
{
  return(&$report_internal([caller],$CONST_OutputModes{Log},"Error: ",@_));
}

sub error_display
{
  return(&$report_internal([caller],$CONST_OutputModes{Display},"Error: ",@_));
}

#------------------------------------------------------------------------------
# The trace function(s) works similarly to the report function except that the 
# header is implicitly "Trace: ".  It only displays if the calling module has
# its debug flag set (or $main::debug evaluates to true).
#
# Note: trace displays to both the log and the screen
#       trace_log displays to only the log
#       trace_display displays to only the screen
#------------------------------------------------------------------------------
sub trace
{
  my @caller_info = caller;
  my $package = $caller_info[0];
  my $debug_symbol = "${package}::debug";

  if ($$debug_symbol || $main::debug) {
    return(&$report_internal(\@caller_info,$CONST_OutputModes{Both},"Trace: ",@_));
  }
}

sub trace_log
{
  my @caller_info = caller;
  my $package = $caller_info[0];
  my $debug_symbol = "${package}::debug";

  if ($$debug_symbol || $main::debug) {
    return(&$report_internal(\@caller_info,$CONST_OutputModes{Log},"Trace: ",@_));
  }
}

sub trace_display
{
  my @caller_info = caller;
  my $package = $caller_info[0];
  my $debug_symbol = "${package}::debug";

  if ($$debug_symbol || $main::debug) {
    return(&$report_internal(\@caller_info,$CONST_OutputModes{Display},"Trace: ",@_));
  }
}

#------------------------------------------------------------------------------
# The note function(s) works similarly to the report function except that the 
# header is implicitly "Note: ".  
#
# Note: note displays to both the log and the screen
#       note_log displays to only the log
#       note_display displays to only the screen
#------------------------------------------------------------------------------
sub note
{
  return(&$report_internal([caller],$CONST_OutputModes{Both},"Note: ",@_));
}

sub note_log
{
  return(&$report_internal([caller],$CONST_OutputModes{Log},"Note: ",@_));
}

sub note_display
{
  return(&$report_internal([caller],$CONST_OutputModes{Display},"Note: ",@_));
}

#------------------------------------------------------------------------------
# The indent function(s) works similarly to the report function except that the 
# header is implicitly the number of spaces specified by the first numeric 
# parameter.
#
# Note: indent displays to only the screen
#       indent_log displays to only the log
#       indent_both displays to both the log and the screen
#------------------------------------------------------------------------------
sub indent 
{
  ($#_ >= 0) || die "Invalid number of arguments.";

  my ($length) = shift @_;
  return(&$report_internal([caller],$length x " ",@_));
}

#------------------------------------------------------------------------------
# Strip the extension from the specified path.
#------------------------------------------------------------------------------
sub strip_extension
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($path) = @_;
  $path =~ s/\.[^\.]*$//;
  
  return($path);
}

#------------------------------------------------------------------------------
# Extract the extension from the specified path.
#------------------------------------------------------------------------------
sub extract_extension
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($path) = @_;
  if (s/(\.[^\.]*)$//) {
    return($1);
  }
  else {
    return("");
  }
}

#------------------------------------------------------------------------------
# Sleep the specified number of microseconds.
#------------------------------------------------------------------------------
sub usleep 
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($sleeptime)=@_;

  select undef,undef,undef,($sleeptime/1000000);
}

#------------------------------------------------------------------------------
# Wait until either any child exits (if no parameters are specified) or 
# the specified child exits (if one parameter, the pid of the child process,
# is specified).
#
# Note: Meant to be used as a parent waiting for a child process to die.
#------------------------------------------------------------------------------
sub spinner_wait
{
  ($#_ == 0) || ($#_ == -1) || die "Invalid number of arguments.";

  my $child = -1;
  my $pid = -1;

  if ($#_ == 0) {
    ($pid) = @_;
  }

  my @SPINLIST = ("|\b", "/\b", "-\b", "\\ \b\b", "|\b", "/\b", "-\b", "\\ \b\b");
  my $length = $#SPINLIST + 1;
  
  do { 
    $child = waitpid(-1,&WNOHANG);
    $SPINCOUNT = ($SPINCOUNT + 1) % $length;
    print("$SPINLIST[$SPINCOUNT]");
    usleep(250000);
  } until(($child == $pid) || ($child == -1));
}

#------------------------------------------------------------------------------
# This is the inverse of spinner_wait.  Namely, it has the child process 
# spinning until the parent process (or any other process, for that matter)
# signals it to no longer spin (via signal_spinner).  
#------------------------------------------------------------------------------
sub spinner_wait_signal
{
  ($#_ == 0) || ($#_ == -1) || die "Invalid number of arguments.";

  my $child = -1;
  my $pid = -1;

  if ($#_ == 0) {
    ($pid) = @_;
  }

  my @SPINLIST = ("|\b", "/\b", "-\b", "\\ \b\b", "|\b", "/\b", "-\b", "\\ \b\b");
  my $length = $#SPINLIST + 1;
  
  do { 
    $SPINCOUNT = ($SPINCOUNT + 1) % $length;
    print("$SPINLIST[$SPINCOUNT]");
    usleep(250000);
  } until(0);  
}

#------------------------------------------------------------------------------
# See spinner_wait_signal above.  Signal the waiting process (kill it).
#------------------------------------------------------------------------------
sub signal_spinner
{
  ($#_ == 0) || die "Invalid number of arguments.";
  my ($pid) = @_;

  kill(&SIGTERM,$pid);
  wait;
  print "\n";
}

#------------------------------------------------------------------------------
# This method will establish the globals used universally throughout the 
# installer.  It should be noted that this method assumes that the
# $pParameters->{asphome} value was set prior to execution.
#------------------------------------------------------------------------------
use INSTALL::global;
use INSTALL::parameter;

my $pParameters = \%INSTALL::parameter::variables;

sub set_standard_globals
{
  (exists $pParameters->{asphome}) || die "Required parameter asphome is not set.";
  (exists $pParameters->{com_layer}) || die "Required parameter com_layer is not set.";
  
  (-d $pParameters->{asphome}) || die "Required parameter asphome refers to an invalid directory.";

  # Strip trailing / off of asphome directory.
  $pParameters->{asphome} =~ s@/+$@@;

  if ($pParameters->{asphome} =~ /^\s*$/) {
    $pParameters->{asphome} = "/";
  }

  my $CSTARGET = "optimized";
  if (exists $ENV{INSTALL_CSTARGET}) {
    $CSTARGET=$ENV{INSTALL_CSTARGET};
  }

  my $COM_LAYER = lc($pParameters->{com_layer});

  %INSTALL::global::variables =
    ( 
     CSTARGET       => $CSTARGET,
     COM_LAYER      => $COM_LAYER,
     com_layer      => $COM_LAYER,

     default_shell  => $pOS->{default_shell},
     
     # Java is known to cause some stability problems on some platforms, 
     # disable it by default.
     java_support  => "no",

     # Set the locale by default to en_US.
     lcid	   => "0409",
     codepage	   => "1252",

     casp_version   => "3.6.0".uc(substr($pOS->{os},0,1)),
     odbc_version   => "15",
     license_name   => "LICENSE.LIC",
     perl           => $pOS->{perl},
     execute_params => join(" ",map { "\"$_\"" } @main::ARGV),
     status_text    => "",
     asphome        => $pParameters->{asphome},
     asp_dir        => $pParameters->{asphome},
     log_dir        => "$pParameters->{asphome}/logs",

     # FIXME: The lib directory should have the capacity to be target dependent too.
     lib_dir        => "$pParameters->{asphome}/lib-$COM_LAYER",

     (($pParameters->{com_layer} =~ /MainWin/i) ?
      (com_lib_dir  => "$pParameters->{asphome}/mainwin/mw/lib-$pOS->{ostag}_$CSTARGET",
       com_bin_dir  => "$pParameters->{asphome}/mainwin/mw/bin-$pOS->{ostag}_$CSTARGET") :
      (com_lib_dir  => "$pParameters->{asphome}/chilicom/lib/$pOS->{ostag}_$CSTARGET",
       com_bin_dir  => "$pParameters->{asphome}/chilicom/bin/$pOS->{ostag}_$CSTARGET")),
      
     module_dir     => "$pParameters->{asphome}/module/$pOS->{ostag}_$CSTARGET",
     install_dir    => "$pParameters->{asphome}/INSTALL",
     templates_dir  => "$pParameters->{asphome}/INSTALL/templates",
     template_dir   => "$pParameters->{asphome}/INSTALL/templates",
     components_dir => "$pParameters->{asphome}/components",
     component_dir  => "$pParameters->{asphome}/components",
     jre_path       => "pPararmeters->{asphome}/bean/jre1.2.2",
     jvm_path       => "pPararmeters->{asphome}/bean/jre1.2.2",
     jre_version    => "1.2.2",
     chili_ini_dir  => "/var/opt/casp",
     chili_ini      => "/var/opt/casp/chili.ini",
     installed_db   => "$pParameters->{asphome}/.installed_db",
     settings_reg   => "$pParameters->{asphome}/INSTALL/settings.reg",
     
     install_log    => "$pParameters->{asphome}/logs/install",
     summary_log    => "$pParameters->{asphome}/logs/install_summary",
     register_log   => "$pParameters->{asphome}/logs/register",
     component_log  => "$pParameters->{asphome}/logs/component_log"
    );

  # Some portions of post install depend on there being an .installed_db.
  open(INSTALLED_DB,">>$INSTALL::global::variables{installed_db}");
  close(INSTALLED_DB);
  chmod(0640,"$INSTALL::global::variables{installed_db}");
  chmod(0777,"$INSTALL::global::variables{log_dir}");
  chmod(0755,"$INSTALL::global::variables{lib_dir}");

  local *LOG;
  open(LOG,">>$INSTALL::global::variables{log_dir}/installer_log");

  LOG->autoflush(1);

#   print LOG 
#     (
#      "===============================================================================\n",
#      "Logging started: ",scalar(localtime(time)),"\n",
#      "===============================================================================\n"
#     );
     

  $ERROR_LOG = *LOG;

  return(1);
}

#------------------------------------------------------------------------------
# Attempt to resolve the location of a binary <name>, using system directories
# before any other.  The optional 2nd, 3rd, etc. arguments specify alternative
# root directories to search.  It should be noted that these directories are 
# prepended to the list to be searched and will thus take precedence over
# default system directories.
#
# Note(0): If no executable path can be found, undef is returned.
# Note(1): The result is the full path name of the binary (not merely the 
#          directory).
#------------------------------------------------------------------------------
sub locate_binary
{
  ($#_ >= 0) || die "Invalid number of arguments.";

  my ($name,@optional_dirs) = @_;
  my $result = undef;

  if ($#optional_dirs >= 0) {
    foreach(@optional_dirs) {
      my $possibility = "$_/$name";
      if (-x $possibility) {
	$hostname = $possibility;
      }      
    }
  }

  foreach("/sbin","/usr/sbin","/usr/local/sbin","/bin",
	  "/usr/bin","/usr/local/bin","/usr/local/bin") 
  {
    my $possibility = "$_/$name";
    if (-x $possibility) {
      $result = $possibility;
    }
  }

  return($result);
}

#------------------------------------------------------------------------------
# Make a directory structure, indepedent of which of the subdirectories of that
# structure already exist.  This is like the user-level command 
# 'mkdir -p <path>'
#------------------------------------------------------------------------------
sub mkpath {
  ($#_ == 1) || die "Invalid number of arguments.";
  my ($path,$mode) = @_;
  my $result = (not -d $path);

  my @subdirs = split(m@/@,$path);
  my $const = "";

  foreach(@subdirs) {
    $const .= "$_/";
      
    if (not -d $const) {
      $result &= mkdir($const,$mode);
    }
  }  
}

#------------------------------------------------------------------------------
# This method returns the realpath of the filename / path specified.  See the
# realpath manpage.  If the file does not exist undef is returned.
#------------------------------------------------------------------------------
sub realpath 
{
  ($#_ == 0) || die "Invalid number of arguments.";
  my ($filename) = @_;    

  # Remove relative path dependency.
  if ($filename =~ m@^[^/]@) {
    # Relative path.
    my $pwd = `pwd`;
    chomp $pwd;

    $filename = "$pwd/$filename";
  }

  if (-e $filename) {
    my $done;
    my @components = split(m@/@,$filename);
    

    my $fullpath;
    foreach my $component (@components) {
      if ($component eq ".") {
	next;
      }
      elsif ($component eq "..") {
	# Strip off the previous directory.
	$fullpath =~ s@/[^/]*/$@/@;
	next;
      }
      
      my $path = "$fullpath$component";

      if (-l $path) {
	my $link = readlink($path);
	
	if ($link =~ m@^[^/]@) {
	  # Relative path.
	  
	  my $parent_dir = $path;
	  $parent_dir =~ s@/[^/]*$@@;
	  
	  $path = realpath("$parent_dir/$link");
	}
	else {
	  $path = realpath($link);
	}
      }

      $fullpath = "$path/";
    }
    
    $fullpath = substr($fullpath,0,length($fullpath)-1);
    
    return($fullpath);
  }
  else {
    return(undef);
  }
}

#------------------------------------------------------------------------------
# Returns the directory name of the specified path (like the Unix dirname
# command).
#
# dirname("/usr/home/deanb") = "/usr/home"
# dirname("/usr/home/deanb/") = "/usr/home"
# dirname("foobar") = "."
# dirname("foobar/fubar") = "foobar"
#------------------------------------------------------------------------------
sub dirname
{
  my @result = @_;

  foreach(@result) {
    s@/$@@;

    if (m@/@) {
      s@/[^/]*$@@;
    }
    else {
      $_ = ".";
    }
  }

  wantarray ? return(@result) : return($result[0]);
}

#------------------------------------------------------------------------------
# Returns the base filename of the specified path (like the Unix basename
# command).
#
# basename("/pub/home/deanb") = "deanb"
# basename("/pub/home/deanb/") = "deanb"
# basename("foobar") = "foobar"
# basename("foobar/fubar") = "fubar"
#------------------------------------------------------------------------------
sub basename
{
  my @result = @_;

  foreach(@result) {
    s@/$@@;

    if (m@/([^/]*)$@) {
      $_ = $1;
    }
  }

  wantarray ? return(@result) : return($result[0]);
}

#------------------------------------------------------------------------------
# Find <find_count> file(s) under the specified <root> directory that matches 
# <filename>.  If <find_count> is left unspecified (or is undef), it is 
# interpreted to mean, find all.
#
# Upon failure, an empty list is returned.
#------------------------------------------------------------------------------
sub find_files
{
  (($#_ == 1) || ($#_ == 2)) || die "Invalid number of arguments.";  
 
  my ($root,$filename,$find_count) = @_;
  my @result;

  if (not defined $find_count) {
    $find_count = 2**31;
  }

  my @subdirs = ($root);
  my $subdir;

  while($subdir = pop @subdirs) {
    local *SUBDIR;
    opendir(SUBDIR,$subdir) || die "Invalid directory: $subdir";
    my @files = readdir(SUBDIR);
    close(ROOT);
    
    foreach(@files) {
      (($_ ne ".") && ($_ ne "..")) || next;

      $find_count || return(@result);
      my $path = "$subdir/$_";
    
      if ($_ eq $filename) {
	push @result, $path;
	$find_count--;
      }
    
      if ((not -l $path) && (-d $path)) {
	push @subdirs, $path;
      }
    }
  }

  wantarray ? return(@result) : return($result[0]);
}

sub find_file
{
  ($#_ == 1) || die "Invalid number of arguments.";  

  my @result = find_files(@_,1);

  wantarray ? return(@result) : return($result[0]);
}

# END 
# {
#   if (defined $ERROR_LOG) {
#     print $ERROR_LOG
#       (
#        "===============================================================================\n",
#        "Logging stopped: ",scalar(localtime(time)),"\n",
#        "===============================================================================\n"
#       );
#   }
# }

1;
