package INSTALL::locale;

#
# locales map
#
# ex. ["literal string", lcid, codepage, default locale (, locales ...)]
#
@locales = (
  ["English - US", "0409", "1252", "en_US", "POSIX", "C"],
  ["English - British", "0809", "1252", "en_GB", "en_UK"],
  ["Japanese Shift-JIS", "0411", "932", "ja", "japanese"],
  ["German", "0407", "1252", "de_DE", "de"],
  ["Dutch", "0413", "1252", "nl_NL", "nl"],
  ["Spanish", "040a", "1252", "es_ES", "es"],
  ["French", "040c", "1252", "fr_FR", "fr"],
  ["Swedish", "041d", "1252", "sv_SV", "sv"],
);

use INSTALL::common;

use INSTALL::global;
use INSTALL::parameter;
use INSTALL::os;
use INSTALL::query;
use INSTALL::conditional;

# This should be set to a value that evaluates to true, if the bean setup
# is wanted.  Otherwise, it is assumed that we will be shipping JRE with
# our product. In both cases, the configure interfaces should 
# be called.

########## Define Globals
my $pGlobals = \%INSTALL::global::variables;
my $pParameters = \%INSTALL::parameter::variables;
my $pOS = \%INSTALL::os::Details;

my $enable_debug = 0;

$debug_output = $enable_debug ? "" : "> /dev/null 2>&1";

my $interactive = (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));

#==============================================================================
# Miscelaneous / Helpers
#==============================================================================

#
# setup 'locales' structure
#
my %locale_hash;
foreach my $pLocale (@locales) {
  $locale_hash{uc($pLocale->[3])} = $pLocale;
}

# 
# see if a locale is available on the current os
# returns 1 if available and 0 if not
#
sub testlocale {

  if ($#_ != 0) {
    return(error("Invalid number of parameters to INSTALL::locale::testlocale [ ",
		 ($#_ + 1), "]."));
  }
  
  my ($locale) = @_;

  for $i (3 .. $#{$locale}) {
    @valid = `locale -a|grep -i $locale->[$i]`;
	for ($j=0; $j<=$#valid; $j++) {
	  chomp $valid[$j];
      if (uc("$valid[$j]") eq uc("$locale->[$i]")) {
        return 1;
      }
    }
  }

  return 0;
}

#
# returns an array of valid locales for the current os
#
sub listlocales {

  my @menuitems;
  foreach my $pLocale (@locales) {
    # check if it is available on the system
    if (testlocale($pLocale)) {
      push @menuitems, $pLocale->[0];
    }
  }
  
  return(@menuitems);
}

#
# returns default locale, defined as the value of
# LANG as returned by the locale function
#
sub default_locale {

  return(substr(`locale | grep LANG`, 5));
}

#
# determine the locale, using different method
# for interactive vs. non-interactive case
#
sub querylocale {

  if ($interactive) {
    return(interactive_querylocale());
  } 
  else {
    return(non_interactive_querylocale());
  }
} 

#
# queries the interactive user for a locale
# returns one of the strings from the list
# above (en_US by default)
#
sub interactive_querylocale {

  my @menuitems;
  my @menuvals;
  foreach my $pLocale (@locales) {
    # check if it is available on the system
    if (testlocale($pLocale)) {
      push @menuitems, $pLocale->[0];
      push @menuvals, $pLocale->[3];
    }
  }

  my $result = INSTALL::query::user_menu
    ("Chili!Soft ASP Language Selection: interactive = $interactive",
     "Chili!Soft ASP can support various languages. Select which ".
     "language you would like to use from the list below.",
     "Which language would you like to use",
     "terminal_mark=?",
     "default=1",
     @menuitems
     );

  return($menuvals[$result - 1]);
}

#
#  returns the default locale on the system
#
sub non_interactive_querylocale {

  return(default_locale());
}




#
# converts a locale string into an LCID value. Returns the converted
# LCID value which defaults to 0409(en_US) on an error
#
sub locale2lcid {
  #---------------------------------------------------------------------------
  # Parameter:			Default value:			Optional:
  #---------------------------------------------------------------------------
  # locale			    <none>				    N

  if ($#_ != 0) {
    return(error("Invalid number of parameters to INSTALL::locale::locale2lcid [ ",
		 ($#_ + 1), "]."));
  }

  my ($locale) = @_;
  my $ret = $locale_hash{uc($locale)};

  if (not defined $ret) {
    return("0409");  
  }
  
  return($ret->[1]);
}

#
# converts a locale string into a CODEPAGE value. Returns the converted
# CODEPAGE value which defaults to 1252(en_US) on an error
#
sub locale2codepage {
  #---------------------------------------------------------------------------
  # Parameter:			Default value:			Optional:
  #---------------------------------------------------------------------------
  # codepage			<none>                  N

  if ($#_ != 0) {
    return(error("Invalid number of parameters to INSTALL::locale::locale2codepage [ ",
		 ($#_ + 1), "]."));
  }

  my ($locale) = @_;
  my $ret = $locale_hash{uc($locale)};

  if (not defined $ret) {
    return("1252");  
  }
  
  return($ret->[2]);  
}

#
# creates an ini file which lists the available locales for the system
#
sub build_locale_cache {
  #---------------------------------------------------------------------------
  # Parameter:			Default value:			Optional:
  #---------------------------------------------------------------------------
  # cachefile			<none>                  N

  if ($#_ != 0) {
    return(error("Invalid number of parameters to INSTALL::locale::build_locale_cache [ ",
		 ($#_ + 1), "]."));
  }
  
  my ($cachefile) = @_;

  $date = `date`;
  system("echo \"# locale cache created on $date\" > $cachefile");

  my @menuitems;
  foreach my $pLocale (@locales) {
    # check if it is available on the system
    if (testlocale($pLocale)) {
      system("echo \"[$pLocale->[0]]\" >> $cachefile");
      system("echo \"lcid=$pLocale->[1]\" >> $cachefile");
      system("echo \"codepage=$pLocale->[2]\" >> $cachefile");
      system("echo \"\" >> $cachefile");
    }
  }
}
1;
