package INSTALL::admin;

use INSTALL::common;
use INSTALL::os;

my $CONST_Package = "INSTALL::admin";
my $CONST_ComponentName = "Administration Console";
my $CONST_StartPriority = 86;
my $CONST_StopPriority = 14;

my $pOS = \%INSTALL::os::Details;

my $debug = 0;
my $debug_output = ($debug ? "" : " > /dev/null 2>&1");

umask 0022;

#------------------------------------------------------------------------------
# If this flag is set to 1, then the installation of the administration console
# is configurable.  Otherwise, it will be installed regardless of what the
# user wants.
#------------------------------------------------------------------------------
my $installation_configurable = 0;

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Internal functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# All of the error reporting mechanisms take of a list of strings, and format
# them such that they fall on different lines.
#------------------------------------------------------------------------------
sub error {
  my $result = INSTALL::common::generate_error(@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}

sub trace
{
  if ($debug)
  {
    my $result = INSTALL::common::generate_report("trace: ",@_);
    print STDERR "Package location: $CONST_Package\n";
    return($result);
  }
}

sub note
{
  my $result = INSTALL::common::generate_note(@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}

sub report 
{
  my $result = INSTALL::common::generate_report("",@_);
  print STDERR "Package location: $CONST_Package\n";
  return($result);
}

use INSTALL::parameter;
use INSTALL::global;
use INSTALL::component;

my $pGlobals = \%INSTALL::global::variables;
my $pParameters = \%INSTALL::parameter::variables;

sub find_binary
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($binary_name) = @_;

  my @possible_dirs = 
    (
     "/bin",
     "/sbin",
     "/usr/bin",
     "/usr/local/bin",
     "/usr/sbin"
    );

  foreach(@possible_dirs) {
    if (-x "$_/$binary_name") {
      return("$_/$binary_name");
    }
  }

  return(undef);
}

sub verify_username
{
  ($username, $password) = (@_);
  
  if ($#_ == 0) {
    return(not system("$pGlobals->{asphome}/admtool -a $username > /dev/null 2>&1"));
  }
  else {
    return(not system("$pGlobals->{asphome}/admtool -a $username $password > /dev/null 2>&1"));
  }
}  

use INSTALL::query;
use INSTALL::conditional;
use INSTALL::services;
use INSTALL::reboot;
use INSTALL::substitute;

#------------------------------------------------------------------------------
# The admin::configure method is responsible for querying / identifying the
# wants of the user and incorporating those wants into the generated console.
#==============================================================================
#------------------------------------------------------------------------------
sub configure
{
  my $compressor;
  my $compressor_name;
  my $tarball;
  my $interactive = 
    (not INSTALL::conditional::boolean_value($pParameters->{non_interactive}));
  
  ########## User query.
  if (not exists $pParameters->{install_admin}) {
    # Default to yes.
    
    $pParameters->{install_admin} = "true";
  }
  
  if ((not exists $pParameters->{admin_port}) || 
      (not $pParameters->{admin_port} =~ /^[0-9]+$/)) 
  {
    # Default to 5100.
    
    $pParameters->{admin_port} = 5100;
  }
  
  if (not exists $pParameters->{admin_startonboot}) {
    # Automatic start enabled by default.
    
    $pParameters->{admin_startonboot} = "true";
  }
  
  if (not exists $pParameters->{admin_username}) {
    # Default username = admin
    $pParameters->{admin_username} = "admin";
  }
  
  if (not exists $pParameters->{admin_password}) {
    # Default password = root
    
    $pParameters->{admin_password} = "root";
  }

  if ($installation_configurable) {
    if (not $interactive) {
      if (not INSTALL::conditional::boolean_value($pParameters->{install_admin})) {
	return(undef);
      }
    }
    else {
      INSTALL::query::user_menu
	("ADMINISTRATION CONSOLE INSTALLATION",
	 "You now have the opportunity to install the Chili!Soft ASP Administration Console.  ".
	 "This utility makes installing Chili!Soft ASP simpler by providing a web-based ".
	 "interface to most commonly needed configuration options.");
      
      my $answer = INSTALL::query::user
	("Would you like to install the administration console",
	 "terminal_mark=?","default=y","n");
      
      if ($answer eq "n") {
	return(undef);
      }    
    }
  }

  if ($interactive) {
    $install_defaults = INSTALL::query::user_menu
      ("Chili!Soft ASP - Administration Console Installation",
       "Setup will now install the Administration Console. You can select the option ".
       "to use the default configuration settings or to specify custom settings. ".
       "NOTE: The user name and password for the Administration Console will ".
       "be set to the default values if you select the Default option. ".
       "Depending on your environment, this may pose a security risk.  ".
       "We recommend that you select the Custom option and specify a user ".
       "name and password to prevent this security hole. ".
       "For more information about configuring the Chili!Soft ASP ".
       "Administration Console, see the online product documentation. ",
       "Select a configuration option for the Administration Console",       
       "terminal_mark=.",
       "default=Default configuration",
       "Custom configuration");
    
    if ($install_defaults == 2) {
      $pParameters->{custom_install} = "true";	
    }
  }
  
  my $custom_install = 
    INSTALL::conditional::boolean_value($pParameters->{custom_install});

  if ($interactive && $custom_install) {
    sub is_port_valid
    {
      my ($check_port) = @_;
      if ($check_port <= 1024) {
	return 0;
      }
      
      return(INSTALL::services::validate_free_ports($check_port,1));
    }
    
    my $recommended = INSTALL::services::locate_free_ports(5100,1);    
    
    INSTALL::query::user_menu
      ("Chili!Soft ASP - Administration Console Options",
       "To configure the Administration Console, please respond to the following prompts.");
    
    $pParameters->{admin_port} = INSTALL::query::user_function
      ("Type the Administration Console port number, or \npress Enter to accept the default",*is_port_valid,"default=$recommended",
       "terminal_mark=.",
       "invalid_choice=Port already in use or invalid");    
    
    print "\n";
  } 
  elsif ($interactive) {
    INSTALL::query::user_menu
      ("Chili!Soft ASP - Configuring the Administration Console",
       "Setup will now perform all of the default option selection for you.\n".
       "Please wait.");
  }
    
  
  if ($interactive && $custom_install) {
    $pParameters->{admin_startonboot} = 
      (INSTALL::query::user
       ("Automatically start the Administration Console on system startup",
	"default=y","n",
	"terminal_mark=?") eq "y") ? "true" : "false";
  }
  
  ########## Install initial tarball into the asphome directory.
  if (-f "$pGlobals->{admin_dir}/admin.tar.Z") {
    $compressor_name = "compress";
    $tarball = "$pGlobals->{admin_dir}/admin.tar.Z";
  }
  elsif (-f "$pGlobals->{admin_dir}/admin.tar.gz") {
    $compressor_name = "gzip";
    $tarball = "$pGlobals->{admin_dir}/admin.tar.gz";
  }
  elsif (-f "$pGlobals->{admin_dir}/admin.tar.bz2") {
    $compressor_name = "bzip2";
    $tarball = "$pGlobals->{admin_dir}/admin.tar.bz2";
  }
  elsif (-f "$pGlobals->{admin_dir}/admin.tar") {
    $compressor_name = "";
    $tarball = "$pGlobals->{admin_dir}/admin.tar";
  }
  else {
    return(error("Unable to find admin tarball."));
  }
  
  # Blow away any unregistered administration console ... god forbid.
  system("/bin/rm -rf $pGlobals->{asphome}/admin $pGlobals->{asphome}/admtool $debug_output");
  
  my $decompressor;
  if ($compressor_name eq "") {
    $decompressor = "cat";
  }
  else {
    $compressor = find_binary($compressor_name);
    
    unless($compressor) {
      return(error("Unable to find compressor binary: $compressor_name."));
    }
    
    $decompressor .= "$compressor -dc";
  }
  
  my $tar = find_binary("tar");
  
  if ($tar) {
    if (system("cd $pGlobals->{asphome}; $decompressor $tarball | $tar -xf -")) {
      return(error("Unable to untar / uncompress $tarball."));
    }
    else {
      $main::ENV{COM_LAYER} = "$pGlobals->{com_layer}";
      $main::ENV{CSREGISTRY} = "$pGlobals->{asphome}/admin/conf";
    }
  }
  else {
    return(error("Unable to find tar binary."));
  }
  
  ########## Perform file substitutions
  my @password_info = getpwnam("nobody");
  my ($username,$password,$uid,$gid);

  if ($#password_info < 0) {
    ($username,$password,$uid,$gid) = getpwnam("chiliasp");
  }
  else {
    ($username,$password,$uid,$gid) = @password_info;
  }

  my ($groupname) = getgrgid($gid);

  if (not $groupname) {
    # If the groups file does not contain an entry for the gid, 
    # merely use the gid as is.
    $groupname = $gid;
  }


  ########## Add to the status_text.
  use Sys::Hostname;
  use Socket;

  my $hostname = hostname;

  my $address = gethostbyname($hostname);
  my $extended_hostname = gethostbyaddr($address,AF_INET);

  if ($extended_hostname) {
    $hostname = $extended_hostname;
  }

  my %subst = 
    ("admin::admd_port" => 
     INSTALL::services::allocate_free_ports($pParameters->{admin_port},1,"Chili!Soft ASP $CONST_ComponentName"),
     "admin::admin_port" => 
     INSTALL::services::allocate_free_ports($pParameters->{admin_port},2,"Chili!Soft ASP $CONST_ComponentName Engines"),
     "admin::webserver_user" => $username,
     "admin::webserver_group" => $groupname,
     "admin::supports_multiprocess" => "false",
     "admin::supports_multiserver" => "false",
     "admin::servername" => $hostname
    );

  foreach($pGlobals->{odbc_version}) {
    /3.6.0/ && 
      do {	
	# All but linux supports multiprocess.
	foreach($pOS->{os}) {
	  (not /Linux/i) && 
	    do {
	      $subst{admin::supports_multiprocess} = "true";
	    };
	}

	last;
      };
    
    # Else, use defaults.
  }
  
  INSTALL::substitute::template("admtool",$pGlobals->{asphome},%subst);
  INSTALL::substitute::template("casp.cnfg","$pGlobals->{asphome}/admin/bin",%subst);
  INSTALL::substitute::template("caspctrl","$pGlobals->{asphome}/admin/bin",%subst);
  INSTALL::substitute::template("apachectl","$pGlobals->{asphome}/admin/bin",%subst);
  INSTALL::substitute::template("admd.conf","$pGlobals->{asphome}/admin/conf",%subst);
  INSTALL::substitute::template("odbc.ini","$pGlobals->{asphome}/admin/conf",%subst);
  INSTALL::substitute::optional_template("Global.asa","$pGlobals->{asphome}/caspsamp","$pGlobals->{asphome}/caspsamp",%subst);
  INSTALL::substitute::optional_template("default.htm","$pGlobals->{asphome}/caspsamp","$pGlobals->{asphome}/caspsamp",%subst);
  INSTALL::substitute::optional_template("setup.asp","$pGlobals->{asphome}/admin/web","$pGlobals->{asphome}/admin/web",%subst);
  
  ########## Duplicate needed executables.
  system("cp $pGlobals->{lib_dir}/caspd $pGlobals->{asphome}/admin/bin/admdog");
  system("cp $pGlobals->{lib_dir}/caspctrlb $pGlobals->{asphome}/admin/bin");
  system("cp $pGlobals->{lib_dir}/caspeng $pGlobals->{asphome}/admin/bin/admeng");

  ########## Register the needed libraries.
  my $ENVIRON="COM_LAYER='$pGlobals->{com_layer}' CSREGISTRY='$pGlobals->{asphome}/admin/conf'";

  my $regsvr = "$ENVIRON $pGlobals->{asphome}/chregsvr";
  my $regedit = "$ENVIRON $pGlobals->{asphome}/chregedit";

  if (not fork) {
    $pGlobals->{register_summary} .= "\nEntering default registry settings (admin-console): Success.\n";
    system("$regedit $pGlobals->{settings_reg} >> $pGlobals->{summary_log} 2>&1");
    
    my @register_libs =
      ((map { "$pGlobals->{com_lib_dir}/lib$_.so" }
	("atl",
	 "vbscript",
	 "jscript")),
       
       (map { "$pGlobals->{lib_dir}/lib$_.so" }
	("chiliado",
	 "chiliscrrun",
	 "calc",
	 "chilicap",
	 "chiliasp")),
               
       (map { "$pGlobals->{asphome}/admin/bin/lib$_.so" }
	("chilifile",
	 "csadmin"))
      );
    
    foreach(@register_libs) {
      my $libname = $_;
      
      if (m@([^/]*)$@) {
	$libname = $1;
      }
      
      $pGlobals->{register_summary} .= "Registering $libname (admin-console): ";
      
      if (not system("$regsvr $_ >> $pGlobals->{install_log} 2>&1")) {
	$pGlobals->{register_summary} .= "Success.";
      }
      else {
	$pGlobals->{register_summary} .= "Failed.";
      }
      
      $pGlobals->{register_log} .= "\n";
    }
  
    if (open(REGISTER,">>$pGlobals->{register_log}")) {
      print REGISTER "$pGlobals->{register_summary}\n";
      close(REGISTER);
    }
    exit(0);
  }
  
  ########## Confirm file modes.
  
  my @binaries = 
    ("$pGlobals->{asphome}/admin/bin/admd",
     "$pGlobals->{asphome}/admin/bin/admdog",
     "$pGlobals->{asphome}/admin/bin/apachectl",
     "$pGlobals->{asphome}/admin/bin/caspctrlb",
     "$pGlobals->{asphome}/admin/bin/admeng",
     "$pGlobals->{asphome}/admin/conf/htpasswd",
     "$pGlobals->{asphome}/admtool");
  
  chmod(0750,@binaries);
  
  ########## Install a user to the administration console.
  if ($interactive && $custom_install) {
    print "\n";
    $pParameters->{admin_username} = 
      INSTALL::query::user_function
	("Type the username for the administrator, or \npress Enter to accept the default.",*verify_username,
	 "default=admin","invalid_choice=Invalid username / password");
  }
  else {
    verify_username($pParameters->{admin_username},$pParameters->{admin_password});
  }
  
  $subst{admin::username} = $pParameters->{admin_username};
  
  ########## Proceed with configuration.
  if (not INSTALL::component::register($CONST_ComponentName,$pGlobals->{asphome})) {
    note("Failed to register administration console.");
  }
  
  if (INSTALL::conditional::boolean_value($pParameters->{admin_startonboot}) &&
      INSTALL::reboot::supported)
  {
    my $script_name = "asp-admin-$subst{admin::admd_port}";
    
    INSTALL::reboot::install_script($script_name,
				    "$pGlobals->{asphome}/admtool -s",
				    "$pGlobals->{asphome}/admtool -e",
				    "$pGlobals->{asphome}/admtool -q",
				    $CONST_StartPriority,$CONST_StopPriority);
  }
  
  ########## Start the console.
  # We must make certain not to start the console, until everything has been registered
  # (ie. by waiting for the child to exit).
  if ($interactive) {
    print STDERR "Finalizing Administration Console configuration ... ";
    INSTALL::common::spinner_wait; 
  }
  else {
    wait;
  }

  if (not fork) {
    system("$pGlobals->{asphome}/admtool -s $debug_output");
    exit(0);
  }

  $pGlobals->{status_text} .=
    ("\n".
     "Administration console installed:\n".
     "  URL: http://$hostname:$subst{admin::admd_port}\n".
     "  Port: $subst{admin::admd_port}\n".
     "  Admin Engine / Daemon ports: $subst{admin::admin_port}\n".
     "  Administration (Root) Username: $subst{admin::username}\n"
    );
  
  if ($interactive) {
    INSTALL::query::user_menu
      ("Chili!Soft ASP - Administration Console Information",
       "Setup has finished installing the Administration Console. To configure ".
       "Chili!Soft ASP, you can connect to the Administration Console from a URL ".
       "or from the command line, as shown below. For more information about configuring ".
       "Chili!Soft ASP, see the online product documentation. \n".
       "It is a good idea to print this page for future reference.");
    
    print "To connect from a browser, use this URL: http://$hostname:$subst{admin::admd_port}\n";
    print "To start, stop and add users, use this script: $pGlobals->{asphome}/admtool\n";
    print "The console's username is: $subst{admin::username}\n";
    print "The console's password is: $pParameters->{admin_password}\n" if (not $custom_install);
    print "\n";
    print "To continue, press Enter.";
    <STDIN>;
  }
  
  my %component_details;
  
  foreach(keys %subst) {
    my $key = $_;
    s/admin:://;
    $component_details{$_} = $subst{$key};
  }
  
  $component_details{uninstaller} =
    ("$pGlobals->{perl} -I$pGlobals->{asphome} -I$pGlobals->{admin_dir} ".
     "$pGlobals->{admin_dir}/install.pl $pGlobals->{execute_params} \"uninstall=true\"");

  ########## Make the service.pwd file readable / writable only by nobody user and
  ########## associated group.
  chown($uid,$gid,"$pGlobals->{asphome}/admin/conf/service.pwd");
  chmod(0600,"$pGlobals->{asphome}/admin/conf/service.pwd");
  
  return(INSTALL::component::merge($CONST_ComponentName,
				   $pGlobals->{asphome},
				   %component_details));
}

#------------------------------------------------------------------------------
# The admin::configure method is responsible for querying / identifying the
# wants of the user and incorporating those wants into the generated console.
#==============================================================================
#------------------------------------------------------------------------------
sub uninstall
{
  my $pComponent = INSTALL::component::open($CONST_ComponentName,
					    $pGlobals->{asphome});

  if (not $pComponent) {
    return(error("Unable to open $CONST_ComponentName component."));
  }

  ########## Stop service
  system("$pGlobals->{asphome}/admtool -e $debug_output");

  ########## Remove reboot scripts, if they exist ...
  if (INSTALL::reboot::supported) {
    my $script_name = "asp-admin-$pComponent->{admd_port}";

    INSTALL::reboot::uninstall_script($script_name,
				      $CONST_StartPriority,
				      $CONST_StopPriority);
  }

  ########## Deallocate used ports.
  INSTALL::services::deallocate_ports($pComponent->{admd_port},1);
  INSTALL::services::deallocate_ports($pComponent->{admin_port},2);

  ########## Remove binaries.
  system("/bin/rm -rf $pGlobals->{asphome}/admtool $pGlobals->{asphome}/admin $debug_output");

  ########## Put the Global.asa of the samples back to pre-admin-console state.
  INSTALL::substitute::file("$pGlobals->{asphome}/caspsamp/default.htm",
			    "$pGlobals->{asphome}/caspsamp/default.htm",
			    { "\"$pComponent->{admd_port}\""  => '"<<admin::admd_port>>"' });
  INSTALL::substitute::file("$pGlobals->{asphome}/caspsamp/Global.asa",
			    "$pGlobals->{asphome}/caspsamp/Global.asa",
			    { "\"$pComponent->{admd_port}\""  => '"<<admin::admd_port>>"' });

  ########## Unregister component.
  INSTALL::component::unregister($CONST_ComponentName,$pGlobals->{asphome});

  $pGlobals->{status_text} .=
    ("\nAdministration console uninstalled.\n");

  return(1);
}

1;
